﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Linq.Expressions;
using System.Reflection;

namespace OrmForSqlBulkCopy
{
	/// <summary>
	/// ORM层.
	/// </summary>
	/// <remarks>因为SqlBulkCopy仅仅是用到少数几个方法, 所以其他的都可不用实现.</remarks>
	/// <typeparam name="TData"></typeparam>
	public class ObjectDataReader<TData> : IDataReader
	{
		private IEnumerator<TData> dataEnumerator;

		private Func<TData, object>[] accessors;

		private Dictionary<string, int> ordinalLookup;

		public ObjectDataReader(IEnumerable<TData> data)
		{
			this.dataEnumerator = data.GetEnumerator();
			var propertyAccessors = typeof(TData)
				.GetProperties(BindingFlags.Instance | BindingFlags.Public)
				.Where(p => p.CanRead)
				.Select((p, i) => new
								{
									Index = i,
									Property = p,
									Accessor = this.CreatePropertyAccessor(p)
								})
				.ToArray();

			this.accessors = propertyAccessors.Select(p => p.Accessor).ToArray();
			this.ordinalLookup = propertyAccessors.ToDictionary(
				p => p.Property.Name,
				p => p.Index,
				StringComparer.OrdinalIgnoreCase);
		}

		/// <summary>
		/// 生成快速获取PropertyInfo上的Value值的表达式
		/// </summary>
		/// <param name="p"></param>
		/// <returns></returns>
		private Func<TData, object> CreatePropertyAccessor(PropertyInfo p)
		{
			var parameter = Expression.Parameter(typeof(TData), "input");

			var propertyAccess = Expression.Property(parameter, p.GetGetMethod());

			var castAsObject = Expression.TypeAs(propertyAccess, typeof(object));

			var lambda = Expression.Lambda<Func<TData, object>>(castAsObject, parameter);
			return lambda.Compile();
		}

		#region IDataReader Members

		public void Close()
		{
			this.Dispose();
		}

		public int Depth
		{
			get
			{
				return 1;
			}
		}

		public DataTable GetSchemaTable()
		{
			return null;
		}

		public bool IsClosed
		{
			get
			{
				return this.dataEnumerator == null;
			}
		}

		public bool NextResult()
		{
			return false;
		}

		public bool Read()
		{
			if (this.dataEnumerator == null)
			{
				throw new ObjectDisposedException("ObjectDataReader");
			}

			return this.dataEnumerator.MoveNext();
		}

		public int RecordsAffected
		{
			get
			{
				return -1;
			}
		}

		#endregion

		#region IDisposable Members

		public void Dispose()
		{
			this.Dispose(true);
			GC.SuppressFinalize(this);
		}

		protected void Dispose(bool disposing)
		{
			if (disposing)
			{
				if (this.dataEnumerator != null)
				{
					this.dataEnumerator.Dispose();
					this.dataEnumerator = null;
				}
			}
		}

		#endregion

		#region IDataRecord Members

		public int FieldCount
		{
			get
			{
				return this.accessors.Length;
			}
		}

		public bool GetBoolean(int i)
		{
			throw new NotImplementedException();
		}

		public byte GetByte(int i)
		{
			throw new NotImplementedException();
		}

		public long GetBytes(int i, long fieldOffset, byte[] buffer, int bufferoffset, int length)
		{
			throw new NotImplementedException();
		}

		public char GetChar(int i)
		{
			throw new NotImplementedException();
		}

		public long GetChars(int i, long fieldoffset, char[] buffer, int bufferoffset, int length)
		{
			throw new NotImplementedException();
		}

		public IDataReader GetData(int i)
		{
			throw new NotImplementedException();
		}

		public string GetDataTypeName(int i)
		{
			throw new NotImplementedException();
		}

		public DateTime GetDateTime(int i)
		{
			throw new NotImplementedException();
		}

		public decimal GetDecimal(int i)
		{
			throw new NotImplementedException();
		}

		public double GetDouble(int i)
		{
			throw new NotImplementedException();
		}

		public Type GetFieldType(int i)
		{
			throw new NotImplementedException();
		}

		public float GetFloat(int i)
		{
			throw new NotImplementedException();
		}

		public Guid GetGuid(int i)
		{
			throw new NotImplementedException();
		}

		public short GetInt16(int i)
		{
			throw new NotImplementedException();
		}

		public int GetInt32(int i)
		{
			throw new NotImplementedException();
		}

		public long GetInt64(int i)
		{
			throw new NotImplementedException();
		}

		public string GetName(int i)
		{
			throw new NotImplementedException();
		}

		public int GetOrdinal(string name)
		{
			int ordinal;
			if (!this.ordinalLookup.TryGetValue(name, out ordinal))
			{
				throw new InvalidOperationException("Unknown parameter name " + name);
			}

			return ordinal;
		}

		public string GetString(int i)
		{
			throw new NotImplementedException();
		}

		public object GetValue(int i)
		{
			if (this.dataEnumerator == null)
			{
				throw new ObjectDisposedException("ObjectDataReader");
			}

			return this.accessors[i](this.dataEnumerator.Current);
		}

		public int GetValues(object[] values)
		{
			throw new NotImplementedException();
		}

		public bool IsDBNull(int i)
		{
			throw new NotImplementedException();
		}

		public object this[string name]
		{
			get
			{
				throw new NotImplementedException();
			}
		}

		public object this[int i]
		{
			get
			{
				throw new NotImplementedException();
			}
		}

		#endregion
	}
}