﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Diagnostics;
using System.Linq;
using System.Text;

namespace OrmForSqlBulkCopy
{
	class Program
	{
		static string sql = @"USE [master]
IF  EXISTS (SELECT name FROM sys.databases WHERE name = N'TempTestDb')
BEGIN
	ALTER DATABASE [TempTestDb] SET  SINGLE_USER WITH ROLLBACK IMMEDIATE
	DROP DATABASE [TempTestDb]
END

CREATE DATABASE [TempTestDb]

USE [TempTestDb]
CREATE TABLE [dbo].[Person](
	[PersonId] [int] IDENTITY(1,1) NOT NULL,
	[Name] [nvarchar](100) NOT NULL,
	[Age] [int] NULL,
	CONSTRAINT [PK_Person] PRIMARY KEY CLUSTERED ([PersonId] ASC)
)";

		static void Main(string[] args)
		{
			var people = CreateSamplePeople(10000);

			using (var connection = new SqlConnection("Server=.;Integrated Security=SSPI"))
			{
				connection.Open();

				var stopwatch = new Stopwatch();

				// ------ SQL Bulk Insert
				// 重置db
				RecreateDatabase(connection);
				InsertDataUsingSqlBulkCopy(people, connection);

				// Measure
				stopwatch.Start();
				InsertDataUsingSqlBulkCopy(people, connection);
				Console.WriteLine("Bulk copy插入: {0}ms", stopwatch.ElapsedMilliseconds);

				// ------ Insert statements
				// 热启动一下
				RecreateDatabase(connection);
				InsertDataUsingInsertStatements(people.Take(2), connection);

				// Measure
				stopwatch.Reset();
				stopwatch.Start();
				InsertDataUsingInsertStatements(people, connection);
				Console.WriteLine("单条插入: {0}ms", stopwatch.ElapsedMilliseconds);

				Console.ReadKey();
			}
		}

		private static void InsertDataUsingInsertStatements(IEnumerable<Person> people, SqlConnection connection)
		{
			using (var command = connection.CreateCommand())
			{
				command.CommandText = "INSERT INTO Person (Name, Age) VALUES (@Name, @Age)";
				var nameParam = command.Parameters.Add("@Name", SqlDbType.NVarChar);
				var dobParam = command.Parameters.Add("@Age", SqlDbType.Int);
				foreach (var person in people)
				{
					nameParam.Value = person.Name;
					dobParam.Value = person.Age;
					command.ExecuteNonQuery();
				}
			}
		}

		private static void InsertDataUsingSqlBulkCopy(IEnumerable<Person> people, SqlConnection connection)
		{
			var bulkCopy = new SqlBulkCopy(connection);
			bulkCopy.DestinationTableName = "Person";
			bulkCopy.ColumnMappings.Add("Name", "Name");
			bulkCopy.ColumnMappings.Add("Age", "Age");

			using (var dataReader = new ObjectDataReader<Person>(people))
			{
				bulkCopy.WriteToServer(dataReader);
			}
		}

		private static IEnumerable<Person> CreateSamplePeople(int count)
		{
			return Enumerable.Range(0, count)
				.Select(i => new Person
				{
					Name = "Person" + i,
					Age = i,
				});
		}


		private static void RecreateDatabase(SqlConnection connection)
		{
			using (var command = connection.CreateCommand())
			{
				command.CommandText = sql;
				command.CommandType = CommandType.Text;
				command.ExecuteNonQuery();
			}
		}
	}

	public class Person
	{
		public string Name { get; set; }

		public int Age { get; set; }
	}
}
